<?php

require_once 'custom/PxRoutines/LogicBuilderPackages/lb62c42db03e09e6_13040435/helpers/date/LbDate_lb62c42db03e09e6_13040435.php';
require_once 'custom/PxRoutines/LogicBuilderPackages/lb62c42db03e09e6_13040435/helpers/date/LbDateTime_lb62c42db03e09e6_13040435.php';

class DateTimeFactory_lb62c42db03e09e6_13040435
{
    public function getDateTimeFromTimestamp($timestamp)
    {
        try {
            global $timedate;
            $date = $timedate->fromTimestamp($timestamp);
            return new LbDateTime_lb62c42db03e09e6_13040435($date);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeFromDbDateString(?string $dateString): ?SugarDateTime
    {
        try {
            if (empty($dateString)) {
                return null;
            }
            global $timedate;
            $date = $timedate->fromDbDate($dateString);
            if (empty($date)) {
                return null;
            }
            $date->setTime(0, 0);
            return new LbDate_lb62c42db03e09e6_13040435($date);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeFromDbDateTimeString(?string $dateString): ?SugarDateTime
    {
        try {
            global $timedate;
            if (empty($dateString)) {
                return null;
            }
            $sugarDate = $timedate->fromDb($dateString);
            if (empty($sugarDate)) {
                return null;
            }
            return new LbDateTime_lb62c42db03e09e6_13040435($sugarDate);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeInCurrentUserTimeZoneFromKnownFormat(string $dateString, string $format): ?SugarDateTime
    {
        try {
            global $current_user;
            return $this->getDateTimeInUserTimeZoneFromKnownFormat($dateString, $format, $current_user);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeInUserTimeZoneFromKnownFormat(string $dateString, string $format, User $user): ?SugarDateTime
    {
        try {
            global $timedate;
            $date = SugarDateTime::createFromFormat($format, $dateString);
            if (empty($date)) {
                return null;
            }
            $timedate->tzUser($date, $user);
            return $this->getLbDateTimeFromSugarDateAndFormatString($date, $format);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeInGmtFromKnownFormat(string $dateString, string $format): ?SugarDateTime
    {
        try {
            $date = SugarDateTime::createFromFormat(
                $format,
                $dateString,
                $this->getGmtTimezone()
            );
            if (empty($date)) {
                return null;
            }
            return $this->getLbDateTimeFromSugarDateAndFormatString($date, $format);
        } catch (Exception $exception) {
            return null;
        }
    }

    public function getDateTimeFromUnknownFormat($dateString): ?SugarDateTime
    {
        if (!is_string($dateString)) {
            return null;
        }
        $date = $this->getDateTimeFromStringUsingCurrentUserDateTimeFormat($dateString);
        if (empty ($date)) {
            $date = $this->getDateTimeFromStringUsingCurrentUserDateFormat($dateString);
        }
        if (empty ($date)) {
            $date = $this->getDateTimeFromStringUsingIsoDateTimeFormat($dateString);
        }
        if (empty ($date)) {
            $date = $this->getDateTimeFromStringUsingIsoDateFormat($dateString);
        }
        if (empty ($date)) {
            $date = $this->getDateTimeInCurrentUserTimezoneFromAnyString($dateString);
        }
        if (empty($date)) {
            return null;
        }
        return $this->getLbDateTimeFromSugarDateTimeAndInitialString($date, $dateString);
    }

    private function getDateTimeFromStringUsingCurrentUserDateTimeFormat(string $dateString): ?SugarDateTime
    {
        global $timedate;
        return $timedate->fromUser($dateString);
    }

    private function getDateTimeFromStringUsingCurrentUserDateFormat(string $dateString): ?SugarDateTime
    {
        global $timedate;
        $date = $timedate->fromUserDate($dateString, false);
        if (empty($date)) {
            return null;
        }
        $date->setTime(0, 0);
        return $date;
    }

    private function getDateTimeFromStringUsingIsoDateTimeFormat(string $dateString): ?SugarDateTime
    {
        global $timedate;
        return $timedate->fromIso($dateString);
    }

    private function getDateTimeFromStringUsingIsoDateFormat(string $dateString): ?SugarDateTime
    {
        global $timedate;
        $date = $timedate->fromIsoDate($dateString);
        if (empty($date)) {
            return null;
        }
        $date->setTime(0, 0);
        return $date;
    }

    private function getDateTimeInCurrentUserTimezoneFromAnyString(string $dateString): ?SugarDateTime
    {
        global $timedate;
        return $timedate->fromString($dateString);
    }

    private function getLbDateTimeFromSugarDateTimeAndInitialString(SugarDateTime $date, string $dateString): SugarDateTime
    {
        if ($this->isDateWithoutTime($dateString)) {
            return new LbDate_lb62c42db03e09e6_13040435($date);
        } else {
            return new LbDateTime_lb62c42db03e09e6_13040435($date);
        }
    }

    private function getLbDateTimeFromSugarDateAndFormatString(SugarDateTime $date, string $format): SugarDateTime
    {
        if ($this->isDateWithoutTimeFormat($format)) {
            return new LbDate_lb62c42db03e09e6_13040435($date);
        } else {
            return new LbDateTime_lb62c42db03e09e6_13040435($date);
        }
    }

    private function isDateWithoutTime(string $dateString): bool
    {
        try {
            $sugarDate = new SugarDateTime(
                $dateString,
                $this->getGmtTimezone()
            );
            if (empty($sugarDate)) {
                return false;
            }
            if (strlen($dateString) < 11) {
                return true;
            }
            return ($sugarDate->getHour() == 0 && $sugarDate->getMinute() == 0 && $sugarDate->getSecond() == 0);
        } catch (Exception $exception) {
            return false;
        }
    }

    private function isDateWithoutTimeFormat(string $format): bool
    {
        $timeSymbols = ['H', 'h', 'G', 'g', 'i', 's', 'U', 'u', 'v'];
        foreach ($timeSymbols as $timeSymbol) {
            if (strpos($format, $timeSymbol)) {
                return true;
            }
        }
        return false;
    }

    private function getDbDateFormat()
    {
        global $timedate;
        return $timedate->get_db_date_format();
    }

    private function getGmtTimezone()
    {
        return new DateTimeZone("UTC");
    }

}
